import { DirectMapObject } from './DirectMap';
type DirectTreeNode<T> = T | DirectTreeObject<T>;
export interface DirectTreeObject<T> {
    [index: string]: DirectTreeNode<T>;
}
export interface DirectTreeBranch<T> {
    parent: DirectTreeObject<T>;
    key: string;
}
export declare namespace DirectTree {
    type Action<T> = (value: T, path: string) => void;
    type Operator<In, Out> = (value: In, ...operands: In[]) => Out;
    const DEFAULT_SEPERATOR = "/";
    /**
     * The reverse-action of Flat, parse a flat tree and build a normal tree.
     * @param seperator seperator used in path
     */
    function Deflat<T>(flatTree: DirectMapObject<T>, seperator?: string): DirectTreeObject<T>;
    /**
     * Delete() deletes the subtree with provided path.
     */
    function Delete<T>(directTree: DirectTreeObject<T>, path: string): void;
    /**
     * Create a flat direct map of leaves with each leaf's path (or path's variant) as it's new key.
     * @param seperator seperator used in path
     */
    function Flat<T>(directTree: DirectTreeObject<T>, seperator?: string): DirectMapObject<T>;
    /**
     * GetBranch() gets the parent of provided path and it's branch key.
     */
    function GetBranch<T>(directTree: DirectTreeObject<T>, path: string): DirectTreeBranch<T> | null;
    /**
     * Get() returns the subtree or leaf with provided path.
     */
    function Get<T>(directTree: DirectTreeObject<T>, path: string): DirectTreeNode<T> | undefined;
    /** IsFlat() determines whether a tree is flat */
    function IsFlat<T>(directTree: DirectTreeObject<T>): boolean;
    /**
     * MergeAt() merges another tree into tree's branch.
     */
    function MergeAt<T>(directTree: DirectTreeObject<T>, path: string, anotherTree: DirectTreeObject<T>): void;
    /**
     * MergeExternal() merges tree and another tree into a new tree.
     */
    function MergeExternal<T>(directTree: DirectTreeObject<T>, anotherTree: DirectTreeObject<T>): DirectTreeObject<T>;
    /**
     * MergeInternal() merges anotherTree into tree.
     */
    function MergeInternal<T>(directTree: DirectTreeObject<T>, anotherTree: DirectTreeObject<T>): void;
    /**
     * OperateLeafInternal() applys provided operator on each leaf value and save the result to a new tree.
     */
    function OperateLeafExternal<In, Out>(directTree: DirectTreeObject<In>, leafOperator: Operator<In, Out>, ...operandTrees: DirectTreeObject<In>[]): DirectTreeObject<Out>;
    /**
     * OperateLeafInternal() applys provided operator on each leaf value directly in source tree.
     */
    function OperateLeafInternal<In>(directTree: DirectTreeObject<In>, leafOperator: Operator<In, In>, ...operandTrees: DirectTreeObject<In>[]): void;
    /**
     * Set() sets the subtree with provided path.
     */
    function Set<T>(directTree: DirectTreeObject<T>, path: string, value: DirectTreeNode<T>): void;
    /**
     * Swap branch of two raw tree.
     */
    function Swap<T>(t1: DirectTreeObject<T>, t2: DirectTreeObject<T>, path: string): boolean;
}
export {};
